function [best_auroc, D] = MC_NonNeg(D0, Y_GroundTruth, lambda, decay_factor, mu_thresh, shldNrm, g, bin_type)

numclasses = size(Y_GroundTruth,1);
trainsize = size(D0,2) - size(Y_GroundTruth,2);

Y_data = D0(1:numclasses,:);


X_data = D0(numclasses+1:end,:);
D0(numclasses+1:end,:) = X_data;

omega_y = find(~isnan(Y_data));
omega_x = find(~isnan(X_data));
omega_x_len = length(omega_x);
omega_y_len = length(omega_y);
best_auroc = 0;

D0_nan = D0;

D0(isnan(D0)) = 0;
[U,sigma,V]  = svds([D0; ones(1,size(D0,2))],1);

%% set parameters
mus = sigma(1)*decay_factor;
while(mus(end) > mu_thresh)
	mus = [mus, mus(end)*decay_factor];
end

tau_z = min(3.8*omega_y_len / (lambda), omega_x_len);

%% initialize with rank-1 projection of D0 with zero on unknown entries
D = sparse(U*sigma*V');

tmp = D(1:end-1,:);
tmp(isnan(D0)) = 0;
D(1:end-1,:) = tmp;

Y = D(1:numclasses,:);
X = D(numclasses+1:end-1,:);

last_err = Inf;
i = 0;

%% main loop 
for mu = mus 
	err_diff = Inf;0
	while(err_diff > 1e-2)
		% gradient step
        if shldNrm == 3
            % use chi-square dist
            gX = 1/omega_x_len * (X.^2 +2*X.*X_data -3*X_data.^2)./((X+X_data).^2);
            gX(isnan(gX)) = 0;
        else        
            % use LS error
            gX = 1/omega_x_len * (X-X_data);
        end

        if bin_type ==1 % -1/1 labels
    		gY = lambda/omega_y_len * (-Y_data./(1+exp(g*Y_data.*Y)));
        else % 0/1 labels
        	gY = lambda/omega_y_len * ((-Y_data)./(1+exp(g*Y_data.*(Y-1))));   
        end

        gZ = [gY;gX];
		gZ(isnan(D0_nan)) = 0; %zero out gradient for unknown entries
		A = D - tau_z *[gZ;zeros(1,size(D0,2))];

		% shrinkage step
		[U,sigma, V] = svd(full(A),'econ');

        Sigma = diag(sigma); r = sum(Sigma > tau_z*mu);
        if r == 0
        %   'div...'
           break
        end
		D = U*diag( max(0,diag(sigma)-tau_z*mu) ) * V';

		% project D back onto feasible region
		D(end,:) = 1;
		D(numclasses+1:end-1,:) = max(0,D(numclasses+1:end-1,:));

		% calculate obj value
		Y = D(1:numclasses,:);
		X = D(numclasses+1:end-1,:);
		
        if bin_type ==1 % -1/1 labels
			label_err = (1/g)*log(1+exp(-g*Y_data.*Y)); % logistic err
        else % 0/1 labels
			label_err = (1/g)*log(1+exp(-g*Y_data.*(Y-1))); % logistic err
        end
		
		if shldNrm == 3
            % use chi-square dist
            feature_err = ((X_data - X).^2) ./(X_data + X); % LS err
            feature_err(isnan(feature_err)) = 0;
		else        
            % use LS error
            feature_err = (X_data - X).^2; % LS err 
		end
       
        nuc_norm = sum(svd(D));
		i = i+1;
		cur_err(i) = mu*nuc_norm + lambda/omega_y_len *sum(label_err(omega_y)) + 1/omega_x_len *sum(feature_err(omega_x));
		
		% check stopping criteria
        if i > 20
    		err_diff = min(abs(cur_err(end) - cur_err(end-10:end-1))./cur_err(end-10:end-1));
        else
            err_diff = abs(cur_err(i) - last_err)/last_err;
        end
		last_err = cur_err(i);
		fprintf('mu=%d, iters = %d, obj=%f, err_diff=%f\n', mu, i, last_err,err_diff)
	end
	if r ~= 0
        [tst_auroc] = auroc(Y(:,trainsize+1:end), Y_GroundTruth);
        if tst_auroc > best_auroc
            best_auroc = tst_auroc;
           	fprintf('MC_NonNeg: mu=%d, lambda = %f, AUROC = %f\n', mu, lambda, tst_auroc);
        end
	end
end
